#!/usr/bin/python3

import sys, time
from ortools.constraint_solver import pywrapcp

def main(chars_total, LT_pair_start):
  print(f"main() {chars_total=} {LT_pair_start=}")

  start_t=time.time()
  solver = pywrapcp.Solver("...")

  # declare variables
  x = [solver.IntVar(0, 26-1, "x%i" % i) for i in range(chars_total)]

  for i in range(chars_total-1):
      if i==LT_pair_start:
          solver.Add(x[i]<x[i+1])
      else:
          solver.Add(x[i]>x[i+1])

  # all distinct
  for i in range(chars_total):
      for j in range(i+1, chars_total):
          solver.Add(x[i]!=x[j])

  solution = solver.Assignment()

  db = solver.Phase(x, solver.CHOOSE_MIN_SIZE_LOWEST_MAX,
                    solver.ASSIGN_MIN_VALUE)

  solver.NewSearch(db)
  solutions = 0
  while solver.NextSolution():
    #print("x: ", [x[i].Value() for i in range(chars_total)])
    solutions+=1
  solver.EndSearch()

  finish_t=time.time()
  print(f"chars_total, LT_pair_start, solutions = {chars_total} {LT_pair_start} {solutions}")
  sys.stdout.flush()
  return solutions

def p(n):
    chars_total=n
    solutions_total=0
    for i in range(chars_total-1):
        solutions_total+=main(chars_total, i)

    print (f"p({n=}) {solutions_total=}")
    return solutions_total

# tests:
assert main(chars_total=26, LT_pair_start=0)==25
assert main(chars_total=26, LT_pair_start=24)==25

assert main(chars_total=26, LT_pair_start=1)==324
assert main(chars_total=26, LT_pair_start=23)==324

assert main(chars_total=26, LT_pair_start=2)==2599
assert main(chars_total=26, LT_pair_start=22)==2599

assert p(3)==10400

#p(6)
#p(25)

#for n in range(26+1):
#    p(n)

"""
def gen_all():
    for chars_total in range(26+1):
        for i in range(chars_total-1):
            log_name="log_%d_%d" % (chars_total, i)
            print ("./PE158.py", chars_total, i, " > ", log_name)

gen_all()
"""

import sys

main(int(sys.argv[1]), int(sys.argv[2]))

